#!/usr/bin/python3

"""
rename-flac takes the information from FLAC metadata to batch rename
the files according to a file-naming scheme.

These are the options you can use to define the filenaming scheme:
  %a = Artist  |  %b = Album        |  %c = Composer  |  %d = Date
  %g = Genre   |  %n = Tracknumber  |  %t = Title
"""
import sys
import subprocess
import re
import os
import argparse


__version__ = "2.2.0"

TAGS = dict(a='artist', b='album', c='composer', d='date',
            g='genre', n='tracknumber', t='title')


def parse_args(sys_args):  # pylint: disable=W0613
    """CLI arguments comprehension"""
    example_text = '''example:
        $ rename-flac "%n - %t" "/media/Main/Musique/The Beatles"'''
    parser = argparse.ArgumentParser(prog='rename-flac',
                                 formatter_class=argparse.RawDescriptionHelpFormatter,
                                 description=__doc__,
                                 epilog=example_text)
    parser.add_argument('scheme', type=str,
                    help='filenaming scheme')
    parser.add_argument('directory', type=str,
                    help='path to the directory containing the album')
    parser.add_argument('--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('--verbose', action='store_true',
                    help='run the program in verbose mode')
    return parser.parse_args()


def scheme_processing(args):
    """Process the scheming arguments."""
    if not re.search("%[tn]", args.scheme):  # To have a unique filename
        raise ValueError("%t or %n has to be present in <scheme>")
    scheme = re.sub('%%([%s])' % ''.join(TAGS.keys()),
                    lambda m: '%%(%s)s' % TAGS[m.group(1)],
                    args.scheme)
    return scheme


def fetch_metadata(filepath):
    """Fetch metadata and format it."""
    mf_args = ["--show-tag=%s" % tag for tag in TAGS.values()]
    tags = ["%s=" % tag for tag in TAGS.values()]
    # make sure the keys exist. Prevents failure in case of incomplete metadata
    metadata = dict.fromkeys(list(TAGS.values()), '')
    pipe = subprocess.Popen(["metaflac"] + mf_args + [filepath],
                            stdout=subprocess.PIPE)
    output, error = pipe.communicate()
    if pipe.returncode:
        raise IOError("metaflac failed: %s" % error)
    output = output.decode('utf-8') # binary to string
    output = output.splitlines()
    for tag in tags:
        for item in output:
            x = re.compile(re.escape(tag), re.IGNORECASE)
            if bool(re.match(x, item)):
                tag = tag.replace("=", "")
                if tag == "tracknumber":
                    metadata[tag] = x.sub("", item).zfill(2)
                else:
                    metadata[tag] = x.sub("", item)
    return metadata


def rename(scheme, dirname, filename, args):
    """Renames the files."""
    filepath = os.path.join(dirname, filename)
    new_filename = scheme % fetch_metadata(filepath) + ".flac"
    if new_filename == filename:
        if args.verbose:
            print('"%s" is already named correctly' % filename)
    else:
        new_filepath = os.path.join(dirname, new_filename)
        os.rename(filepath, new_filepath)
        if args.verbose:
            print('"%s" --> "%s"' % (filename, new_filename))


def main():
    """Main function."""
    args = parse_args(sys.argv[1:])
    scheme = scheme_processing(args)
    if not os.path.isdir(args.directory):
        raise NotADirectoryError("%s is not a valid directory"
                                 % args.directory)
    counter = 0
    for dirname, _, filenames in os.walk(args.directory, topdown=False):
        for filename in sorted(filenames):
            if filename.endswith('.flac'):
                counter += 1
                try:
                    rename(scheme, dirname, filename, args)
                except KeyboardInterrupt:
                    raise
                except OSError:
                    sys.exit("Error: %s title contains /. Please rename it"
                             % filename)
    if counter < 1:
        raise FileNotFoundError("No FLAC files found in %s"
                                 % args.directory)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""Setup"""
from rename_flac import __version__
from setuptools import setup

setup(
    name='rename-flac',
    version=__version__,
    description="""A command-line tool that takes the information from FLAC
                metadata to batch rename the files according to a filenaming
                scheme.""",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Louis-Philippe Véronneau',
    url='https://gitlab.com/baldurmen/rename-flac',
    py_modules = ['rename_flac'],
    entry_points={
        'console_scripts': [
            'rename-flac = rename_flac:main',
        ]
    },
    test_suite="tests",
    license='GPLv3+',
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3',
        'Topic :: Multimedia :: Sound/Audio',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
    ],
)



#!/usr/bin/python3

"""
Basic unit tests for rename-flac
"""

import unittest

from types import SimpleNamespace
from rename_flac import scheme_processing, fetch_metadata, rename


class TestScheme(unittest.TestCase):
    """Test the scheme processor"""

    def test_tracknumber_title(self):
        args = SimpleNamespace(scheme='%n - %t')
        scheme = scheme_processing(args)
        self.assertEqual(scheme, '%(tracknumber)s - %(title)s')

    def test_tracknumber_title_extra(self):
        args = SimpleNamespace(scheme='%n - %t Foobars200')
        scheme = scheme_processing(args)
        self.assertEqual(scheme, '%(tracknumber)s - %(title)s Foobars200')

    def test_all(self):
        args = SimpleNamespace(scheme='%a - %b - %c - %d - %g - %n - %t')
        scheme = scheme_processing(args)
        self.assertEqual(scheme, "%(artist)s - %(album)s - %(composer)s - "
                                 "%(date)s - %(genre)s - %(tracknumber)s - "
                                 "%(title)s")

    def test_no_tracknumber_or_title(self):
        args = SimpleNamespace(scheme='%a')
        with self.assertRaises(ValueError):
            scheme = scheme_processing(args)


class TestMetadata(unittest.TestCase):
    """Basic metadata tests"""

    def test_metadata(self):
        metadata = fetch_metadata('tests/blank.flac')
        self.assertEqual(metadata['artist'], 'Artist')
        self.assertEqual(metadata['album'], 'Album')
        self.assertEqual(metadata['composer'], 'Composer')
        self.assertEqual(metadata['date'], 'Date')
        self.assertEqual(metadata['genre'], 'Genre')
        self.assertEqual(metadata['tracknumber'], 'Tracknumber')
        self.assertEqual(metadata['title'], 'Title')


if __name__ == '__main__':
    unittest.main()
